@@ -24,7 +24,12 @@ class MqttDevice:
         # Get logger
         self.logger = logger.getChild(name)
         self._name = name
-        self.mqtt_root_topic = os.environ.get("ROOT_TOPIC", "homeassistant")
+        self.mqtt_discovery_root_topic = os.environ.get(
+            "MQTT_DISCOVERY_ROOT_TOPIC", os.environ.get("ROOT_TOPIC", "homeassistant")
+        )
+        self.mqtt_data_root_topic = os.environ.get(
+            "MQTT_DATA_ROOT_TOPIC", "homeassistant"
+        )
         self._entities = []
         self._model = None
         self._manufacturer = None
@@ -49,6 +54,7 @@ class MqttDevice:
         name,
         entity_settings,
         subscriptions=None,
+        sub_mqtt_topic=None,
     ):
         """Add a new entity in the device."""
         if entity_type.lower() not in ENTITY_TYPES:
@@ -58,10 +64,18 @@ class MqttDevice:
             self.logger.error(msg)
             raise MQTTHassBaseError(msg)
 
+        if sub_mqtt_topic:
+            mqtt_data_root_topic = "/".join(
+                (self.mqtt_data_root_topic, sub_mqtt_topic.strip("/"))
+            )
+        else:
+            mqtt_data_root_topic = self.mqtt_data_root_topic
+
         self.logger.info("Adding entity %s - %s", entity_type, name)
         ent = getattr(mqtt_entity, "Mqtt" + entity_type.capitalize())(
             name=name,
-            mqtt_root_topic=self.mqtt_root_topic,
+            mqtt_discovery_root_topic=self.mqtt_discovery_root_topic,
+            mqtt_data_root_topic=mqtt_data_root_topic,
             logger=self.logger,
             device_payload=self.config_device_payload,
             subscriptions=subscriptions,
